@@ -34,19 +34,7 @@ unless (defined &{"utf8::is_utf8"}) {
     filehandle with an encoding, see open and perlfunc/binmode.
 =cut
 eval { binmode STDOUT, ':utf8' }; # Fails for perl 5.6
-diag("Can't set binmode(STDOUT, ':utf8'): $@") if $@;
-eval { binmode STDERR, ':utf8' }; # Fails for perl 5.6
-diag("Can't set binmode(STDERR, ':utf8'): $@") if $@;
-
-# Test::More duplicates STDOUT/STDERR at the start but does not copy the IO
-# layers from our STDOUT/STDERR. As a result any calls to Test::More::diag
-# with utf8 data will show warnings. Similarly, if we pass utf8 into
-# Test::More::pass, ok, etc etc. To get around this we specifically tell
-# Test::More to use our newly changed STDOUT and STDERR for failure_output
-# and output.
-my $tb = Test::More->builder;
-binmode($tb->failure_output, ':utf8');
-binmode($tb->output, ':utf8');
+print "Can't set binmode(STDOUT, ':utf8'): $@" if $@;
 
 sub long_test_cols
 {
@@ -153,24 +141,13 @@ sub test_data
     return $test_data;
 }
 
-sub oracle_test_dsn
-{
-    my( $default, $dsn ) = ( 'dbi:Oracle:', $ENV{ORACLE_DSN} );
-    
-    
-    $dsn ||= $ENV{DBI_DSN} if $ENV{DBI_DSN} && ($ENV{DBI_DSN} =~ /^$default/io);
-    $dsn ||= $default;
-    
-    return $dsn;
-}
-
 sub db_handle
 {
-    my $dsn = oracle_test_dsn();
+
     my $dbuser = $ENV{ORACLE_USERID} || 'scott/tiger';
-    my $dbh = DBI->connect($dsn, $dbuser, '', {
+    my $dbh = DBI->connect('dbi:Oracle:', $dbuser, '', {
         AutoCommit => 1,
-        PrintError => 0,
+        PrintError => 1,
         ora_envhp  => 0, # force fresh environment (with current NLS env vars)
     });
     return $dbh;
@@ -187,9 +164,8 @@ sub show_test_data
 	my $v = $$recR[0];
         my $byte_string = byte_string($v);
         my $nice_string = nice_string($v);
-        my $out = sprintf( "row: %3d: nice_string=%s byte_string=%s (%s, %s)\n",
-                           $cnt, $nice_string, $byte_string, $v, DBI::neat($v));
-        note($out);
+        printf( "row: %3d: nice_string=%s byte_string=%s (%s, %s)\n",
+                $cnt, $nice_string, $byte_string, $v, DBI::neat($v));
     }
     return $cnt;
 }
@@ -363,7 +339,7 @@ sub cmp_ok_byte_nice {
     return $ok1 && $ok2;
 }
 
-sub create_table
+sub create_table 
 {
     my ($dbh,$tdata,$drop) = @_;
     my $tcols = $tdata->{cols};
@@ -382,12 +358,10 @@ sub create_table
         $dbh->do(qq{ drop table $table });
         warn "Unexpectedly had to drop old test table '$table'\n" unless $dbh->err;
         $dbh->do($sql);
-    } elsif ($dbh->err) {
-        return;
     } else {
        #$sql =~ s/ \( */(\n\t/g;
        #$sql =~ s/, */,\n\t/g;
-       note("$sql\n") ;
+       print "$sql\n" ;
     }
     return $table;
 #    ok( not $dbh->err, "create table $table..." );
@@ -397,20 +371,18 @@ sub create_table
 
 sub show_db_charsets
 {
-    my ( $dbh) = @_;
-    my $out;
+    my ( $dbh, $fh ) = @_;
+    $fh ||= \*STDOUT;
     my $ora_server_version = join ".", @{$dbh->func("ora_server_version")||[]};
     my $paramsH = $dbh->ora_nls_parameters();
-    $out = sprintf "Database $ora_server_version CHAR set is %s (%s), NCHAR set is %s (%s)\n",
+    printf $fh "Database $ora_server_version CHAR set is %s (%s), NCHAR set is %s (%s)\n",
 	$paramsH->{NLS_CHARACTERSET}, 
 	db_ochar_is_utf($dbh) ? "Unicode" : "Non-Unicode",
 	$paramsH->{NLS_NCHAR_CHARACTERSET},
 	db_nchar_is_utf($dbh) ? "Unicode" : "Non-Unicode";
-    note($out);
     my $ora_client_version = ORA_OCI();
-    $out = sprintf "Client $ora_client_version NLS_LANG is '%s', NLS_NCHAR is '%s'\n",
+    printf $fh "Client $ora_client_version NLS_LANG is '%s', NLS_NCHAR is '%s'\n",
 	ora_env_var("NLS_LANG") || "<unset>", ora_env_var("NLS_NCHAR") || "<unset>";
-    note($out);
 }
 sub db_ochar_is_utf { return shift->ora_can_unicode & 2 }
 sub db_nchar_is_utf { return shift->ora_can_unicode & 1 }
@@ -440,28 +412,22 @@ sub set_nls_nchar
     } else {
         undef $ENV{NLS_NCHAR}; # XXX windows? (perhaps $ENV{NLS_NCHAR}=""?)
     }
-    # Special treatment for environment variables under Cygwin -
-    # see comments in dbdimp.c for details.
-    DBD::Oracle::ora_cygwin_set_env('NLS_NCHAR', $ENV{NLS_NCHAR}||'')
-	if $^O eq 'cygwin';
-    note(defined ora_env_var("NLS_NCHAR") ?	# defined?
+    print defined ora_env_var("NLS_NCHAR") ?	# defined?
         "set \$ENV{NLS_NCHAR}=$cset\n" :
-        "set \$ENV{NLS_LANG}=undef\n")		# XXX ?
+        "set \$ENV{NLS_LANG}=undef\n"		# XXX ?
             if defined $verbose;
 }
 
 sub set_nls_lang_charset
 {
     my ($lang,$verbose) = @_;
-
-    $ENV{NLS_LANG} = $lang ? "AMERICAN_AMERICA.$lang" : '';
-
-    note "set \$ENV{NLS_LANG='$ENV{NLS_LANG}'";
-
-    # Special treatment for environment variables under Cygwin -
-    # see comments in dbdimp.c for details.
-    DBD::Oracle::ora_cygwin_set_env('NLS_LANG', $ENV{NLS_LANG}||'')
-        if $^O eq 'cygwin';
+    if ( $lang ) {
+        $ENV{NLS_LANG} = "AMERICAN_AMERICA.$lang";
+        print "set \$ENV{NLS_LANG}=AMERICAN_AMERICA.$lang\n" if ( $verbose );
+    } else {
+        $ENV{NLS_LANG} = "";	# not the same as set_nls_nchar() above which uses undef
+        print "set \$ENV{NLS_LANG}=''\n" if ( $verbose );
+    }
 }
 
 sub byte_string {
@@ -521,3 +487,4 @@ select $cols from $table;
 
 
 1;
+
